<?php

if( isset($_POST['install_path']) && __soy2_check_token() ){
	$install_path = __soy2_realpath($_POST['install_path']);
	$config = $install_path . "common/common.inc.php";

	if(file_exists($config)){

		include_once($config);

		outputHeader();
		execute();
		outputFooter();

		exit;

	}else{
		$error = true;
	}
}

$install_path = __search_soycms_install_dir();
$config = $install_path . "common/common.inc.php";

outputHeader();
if(@$error)outputError();
outputBody();
outputFooter();

/* --------------------------------------------------------------------------------------- */

function outputBody(){
	global $install_path;
	$path = htmlspecialchars($install_path,ENT_QUOTES,"UTF-8");
	$token = __soy2_get_token();

echo <<<HTML
<form method="post">
	<input type="hidden" name="soy2_token" value="$token" />
	<p>SOY CMSのインストール先を選択してください。</p>
	<input type="text" name="install_path" value="$path" size="120" />
	<p>
		<input type="submit" value="バージョンアップ実行" />
	</p>
</form>

<ul>
<li>「SOY CMSのインストール先」にはadmin, common, soycmsというディレクトリの親ディレクトリを指定してください。</li>
</ul>
HTML;
}

function outputError(){
	echo "<p style='color:red'>指定された場所（".htmlspecialchars($_POST['install_path'],ENT_QUOTES,"UTF-8")."）にはSOY CMSがインストールされていません。</p>";
}


function outputHeader(){
	echo <<<HEADER_HTML
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html lang="ja">

<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
<title>SOYCMS Version Up Program</title>
</head>
<body>
<h1>SOYCMS バージョンアッププログラム(1.2.7 -> 1.3.0)</h1>
HEADER_HTML;
}

function outputFooter(){
	echo <<<FOOTER_HTML
</body>
</html>
FOOTER_HTML;
}


/**
 * バージョンアップ実行
 */
function execute(){

	$failed = array();
	$checkSQL = "select app_role_config from AppRole;";
	$checkSiteSQL = "select not_existing_column from soycms_data_sets;";

	switch(SOYCMS_DB_TYPE){
		case "mysql":
			$adminSql = array(
				"ALTER TABLE AppRole ADD COLUMN app_role INTEGER default 1;",
				"ALTER TABLE AppRole ADD COLUMN app_role_config TEXT;",
				"ALTER TABLE Site ADD COLUMN site_type INTEGER default 1 after site_id;",
				"CREATE table soycms_admin_data_sets(
				  id INTEGER PRIMARY KEY AUTO_INCREMENT,
				  class_name VARCHAR(255) unique,
				  object_data LONGTEXT
				) TYPE = InnoDB;",
				"INSERT INTO soycms_admin_data_sets (class_name, object_data) VALUES ('SOYCMS_DB_VERSION', '1.3');",
				"CREATE TABLE CookieLogin (
				  id INTEGER PRIMARY KEY AUTO_INCREMENT,
				  user_id INTEGER NOT NULL,
				  token VARCHAR(255) UNIQUE NOT NULL,
				  expire INTEGER
				) TYPE = InnoDB;",
			);
			$siteSql = array(
				"drop table soycms_data_sets",
				"create table soycms_data_sets(id integer primary key AUTO_INCREMENT,class_name varchar(255) unique,object_data LONGTEXT)",
			);

			break;
		case "sqlite":
			$adminSql = array(
				"ALTER TABLE AppRole ADD COLUMN app_role INTEGER default 1;",
				"ALTER TABLE AppRole ADD COLUMN app_role_config TEXT;",
				"ALTER TABLE Site ADD COLUMN site_type INTEGER default 1;",
				"CREATE TABLE soycms_admin_data_sets(
				  id INTEGER PRIMARY KEY,
				  class_name VARCHAR UNIQUE,
				  object_data TEXT
				);",
				"INSERT INTO soycms_admin_data_sets (class_name, object_data) VALUES ('SOYCMS_DB_VERSION', '1.3');",
				"CREATE TABLE CookieLogin (
				  id INTEGER PRIMARY KEY,
				  user_id INTEGER NOT NULL,
				  token VARCHAR UNIQUE NOT NULL,
				  expire INTEGER
				);",
			);
			$siteSql = array(
				"",
			);
			break;
	}


	/**
	 * CMS (admin)のDB
	 */
	if(count($adminSql)){
		echo "<p>管理側のデータベースをバージョンアップします。</p>";

		$pdo = new PDO(ADMIN_DB_DSN,ADMIN_DB_USER,ADMIN_DB_PASS,array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION));

		//実行
		try{
			//常に更新を試みる
			throw new Exception("Upgrade");

			//チェック
			$pdo->exec($checkSQL);
			echo "<p>管理側のデータベースはすでに更新されています。バージョンアップをスキップします。</p>";
		}catch(Exception $e){
			foreach($adminSql as $query){
				if(strlen(trim($query)) > 0){
					try{
						$pdo->exec($query);
					}catch(Exception $e){
						$failed[] .= "[admin]\n".$e->getMessage()."\n\n";
					}
				}
			}
			echo "<p>管理側のデータベースをバージョンアップしました。</p>";
		}

	}


	/**
	 * サイトのDB
	 */
	if(count($siteSql)){
		echo "<p>各サイトのデータベースをバージョンアップします。</p>";

		//サイトを取得
		SOY2DAOConfig::Dsn(ADMIN_DB_DSN);
		SOY2DAOConfig::user(ADMIN_DB_USER);
		SOY2DAOConfig::pass(ADMIN_DB_PASS);

		$pdo = SOY2DAO::_getDataSource();
		$siteDAO = SOY2DAOFactory::create("admin.SiteDAO");
		$sites = $siteDAO->get();

		foreach($sites as $key => $site){
			echo "<p>".$site->getSiteName() . "のデーターベースをバージョンアップします。</p>";

			//接続
			try{
				switch(SOYCMS_DB_TYPE){
					case "mysql":
						$pdo = new PDO($site->getDataSourceName(),ADMIN_DB_USER,ADMIN_DB_PASS,array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION));
						break;
					case "sqlite":
					default:
						if(!is_writable($site->getPath().".db/sqlite.db")){
							$failed[] .= "このファイルへの書き込み権限はありません。".$site->getPath().".db/sqlite.db"."\n\n";
						}
						$pdo = new PDO("sqlite:".$site->getPath().".db/sqlite.db","","",array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION));
						break;
				}
			}catch(Exception $e){
				if(!is_writable($site->getPath().".db/sqlite.db")){
					$failed[] .= $site->getSiteName() . "のデーターベースをオープン出来ませんでした。\n\n";
				}
				continue;
			}

			//実行
			try{
				//常に更新を試みる
				throw new Exception("Upgrade");

				//チェック
				$pdo->exec($checkSiteSQL);
				echo "<p>".$site->getSiteName() . "のデータベースはすでに更新されています。スキップします。</p>";
			}catch(Exception $e){
				foreach($siteSql as $query){
					if(strlen(trim($query)) > 0){
						try{
							$pdo->exec($query);
						}catch(Exception $e){
							$failed[] .= $site->getSiteName()."\n".$e->getMessage()."\n\n";
						}
					}
				}
				echo "<p>".$site->getSiteName() . "のデータベースをバージョンアップしました。"."</p>";
			}

		}
	}


	/*=======================================*/

	echo "<p>バージョンアップが完了しました。</p>";
	echo "<p>このバージョンアッププログラムをサーバーから削除してください。</p>";
	if(count($failed)){
		echo "<p>";
		echo "エラーが".count($failed)."件ありました。<br/>";
		echo "すでにバージョンアップを実行済みの可能性があります。";
		echo "</p>";
		echo "<p>エラーログ</p>";
		foreach($failed as $value){
			echo "<pre>";
			echo htmlspecialchars($value,ENT_QUOTES,"UTF-8");
			echo "</pre>";
		}
	}else{
		echo "<p>エラーがはありません。</p>";
	}

}


function __soy2_realpath($dir){

	$path = realpath($dir);

	if(!$path)return $path;

	$path = str_replace("\\","/",$path);
	if(is_dir($path) && $path[strlen($path)-1] != "/")$path .= "/";

	return $path;
}

function __soy2_get_token(){

	if(!isset($_SESSION))@session_start();

	if(!isset($_SESSION["soy2_token"])){
		$_SESSION["soy2_token"] = __soy2_generate_token();
	}

	return $_SESSION["soy2_token"];

}
function __soy2_check_token(){

	if(!isset($_SESSION))@session_start();

	if(isset($_SESSION["soy2_token"]) AND isset($_REQUEST["soy2_token"])){
		if($_REQUEST["soy2_token"] === $_SESSION["soy2_token"]){
			$_SESSION["soy2_token"] = __soy2_generate_token();
			return true;
		}
	}


	return false;

}
function __soy2_generate_token(){
	return md5(mt_rand());
}

function __search_soycms_install_dir(){
	if(file_exists(__soy2_realpath(dirname(dirname(__FILE__)))."common/common.inc.php")){
		$install_path = __soy2_realpath(dirname(dirname(__FILE__)));
	}elseif(file_exists(__soy2_realpath(dirname(__FILE__)."common/common.inc.php"))){
		$install_path = __soy2_realpath(dirname(__FILE__));
	}elseif(file_exists(__soy2_realpath($_SERVER["DOCUMENT_ROOT"])."cms/common/common.inc.php")){
		$install_path = __soy2_realpath($_SERVER["DOCUMENT_ROOT"])."cms/";
	}elseif(file_exists(__soy2_realpath($_SERVER["DOCUMENT_ROOT"])."soycms/common/common.inc.php")){
		$install_path = __soy2_realpath($_SERVER["DOCUMENT_ROOT"])."soycms/";
	}elseif(file_exists(__soy2_realpath($_SERVER["DOCUMENT_ROOT"])."common/common.inc.php")){
		$install_path = __soy2_realpath($_SERVER["DOCUMENT_ROOT"]);
	}else{
		$install_path = __soy2_realpath($_SERVER["DOCUMENT_ROOT"]);
	}

	return $install_path;

}
